#!/usr/bin/perl -w
# Chequeo pasivo generico
# Lee los servicios a chequear de /etc/nagios/check_nsca.conf
# Sustituye las variables $VAR$ por los valores pasados por linea de comandos
#
#
# 1.1: 2014-05-14 daniel
#      Uso &print_version para version
#      Agrego opcion -P para el puerto del NSCA
#      Agrego chequeo 'Uptime' con el uptime de la maquina
#
# 1.0: 2014-05-05 daniel
#      Version inicial

use strict;
use Net::SNMP qw(:snmp);
use Data::Dumper;
use Getopt::Long;
use lib "/usr/lib64/nagios/plugins";
use utils qw($TIMEOUT %ERRORS);
use ipsupervisor qw(&send_nsca &round &check_drbd &read_config &print_version
                    $nagios_hostname $nsca_host $nsca_port $debug $snmp_timeout);

my ($opt_h, $opt_V, $opt_v, $opt_N, $opt_H, $opt_P, $opt_C, $opt_S, $opt_d);
my $PROGNAME = $0;

sub print_usage();

&read_config();
$opt_H = $nagios_hostname if $nagios_hostname;
$opt_N = $nsca_host if $nsca_host;
$opt_P = $nsca_port if $nsca_port;

Getopt::Long::Configure('bundling');
GetOptions
    ("h"   => \$opt_h, "help"           => \$opt_h,
     "V"   => \$opt_V, "version"        => \$opt_V,

     "N=s" => \$opt_N, "nsca=s"         => \$opt_N,
     "H=s" => \$opt_H, "hostname=s"     => \$opt_H,
     "P=s" => \$opt_P, "port=s"         => \$opt_P,
     "C=s" => \$opt_C, "community=s"    => \$opt_C,
     "v=s" => \$opt_v, "snmp_version=s" => \$opt_v,
     "S=s" => \$opt_S, "snmp=s"         => \$opt_S,
     "d"   => \$opt_d, "debug"          => \$opt_d);

if ($opt_V) {
    print_version($PROGNAME,'1.1');
    exit $ERRORS{'OK'};
}

if ($opt_h) {
	print_usage();
	exit $ERRORS{'OK'};
}

# Hostname Centreon (requerido)
$opt_H = shift unless ($opt_H);
(print("Nagios hostname required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_H);

# NCSA Host (requerido)
$opt_N = shift unless ($opt_N);
(print("NCSA host required!\n") && print_usage() && exit $ERRORS{'UNKNOWN'}) unless ($opt_N);

## Opcionales
# NSCA Port
$opt_P = shift unless ($opt_P);
if ($opt_P) {
    if ($opt_P =~ /^(\d+)$/) {
	$nsca_port = $1 ;
    } else {
	print("Invalid NSCA port: $opt_P\n") && exit $ERRORS{'UNKNOWN'};
    }
}

# Host SNMP
($opt_S) || ($opt_S = shift) || ($opt_S = "localhost");

# Community
($opt_C) || ($opt_C = shift) || ($opt_C = "ipcontact");

# Version SNMP
($opt_v) || ($opt_v = shift) || ($opt_v = "1");
my $snmp = $1 if ($opt_v =~ /(\d)/);

# Debug
$opt_d = shift unless ($opt_d);
$debug = $opt_d;

my $line;
my @chequeos;
my ($service,$check_cmd);
open(CONFIG, "/etc/nagios/check_nsca.conf") || die "Error al abrir configuracion!\n";
while ($line = <CONFIG>) {
    next if ($line =~ /^\s*\#\w*/);
    ($service, $check_cmd) = ($1,$2) if ($line =~ /^(\w+)\|(.+)$/);
    # Sustituyo variables
    $check_cmd =~ s/\$HOSTADDRESS\$/$opt_S/g;
    $check_cmd =~ s/\$HOSTSNMPCOMMUNITY\$/$opt_C/g;
    $check_cmd =~ s/\$HOSTSNMPVERSION\$/$opt_v/g;
    my %serv = (
	cmd => $check_cmd,
	service => $service
    );
    push @chequeos,\%serv;
    #print "service: $service, cmd: $check_cmd\n";
}
close CONFIG;

#print Dumper @chequeos;    

my ($cmd, $res, $status);
my $nsca_str;
foreach my $c (@chequeos) {
    # $USER1$/check_centreon_snmp_loadaverage -H $HOSTADDRESS$ -v $_HOSTSNMPVERSION$ -C $_HOSTSNMPCOMMUNITY$ -w $ARG1$ -c $ARG2$che
    #print "* Chequeo: " . $c->{service} . "\n";
        $cmd = "/usr/lib64/nagios/plugins/" . $c->{cmd};
    $res = `$cmd`;
    $status = $? >> 8;
    #print "Res: $res, status $status\n";
    $nsca_str = "$opt_H|" . $c->{service} . "|" . $status . "|$res\n";
    &send_nsca($nsca_str, $opt_N);
}



sub print_usage () {
    print "\nUsage:\n";
    print "$PROGNAME\n";
    print "   -N (--nsca)          NCSA host to send passive check (required)\n";
    print "   -H (--hostname)      Nagios hostname (required)\n";
    print "   -P (--port)         NSCA port to send passive check (default 5667)\n";
    print "   -S (--snmp)          SNMP hostname to query - (defaults to 'localhost')\n";
    print "   -C (--community)     SNMP read community (defaults to 'ipcontact',\n";
    print "                        used with SNMP v1 and v2c\n";
    print "   -v (--snmp_version)  1 for SNMP v1 (default)\n";
    print "                        2 for SNMP v2c\n";
    print "   -d (--debug)         Enable NSCA debug (default disabled)\n";
    print "   -V (--version)       Plugin version\n";
    print "   -h (--help)          usage help\n";

}

